using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace NETSploit
{
    public partial class MainForm : Form
    {
        ParseItemXML currentItem;
        public MainForm()
        {
            InitializeComponent();
            ParseConfigXML.ParseFile("Config.xml");
            SetConfig();
        }

        private void SetConfig()
        {
            copyFromGACToolStripMenuItem.Checked = Configuration.copyFromGAC;
            promptForDeployerToolStripMenuItem.Checked = Configuration.promptForDeployer;
        }

        private void DisplayProgress(string s)
        {
            progresslistBox.Items.Add(s);
            Update();
        }

        private void ClearDisplay()
        {
            progresslistBox.Items.Clear();
            ReferenceslistBox.Items.Clear();
            FunctionslistBox.Items.Clear();
            PayloadslistBox.Items.Clear();
            Update();
        }

        private void loadItemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ClearDisplay();
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.InitialDirectory = Configuration.ItemsDir;
            dlg.RestoreDirectory = true;
            dlg.Filter = "Item Files (*.item)|*.item|All files (*.*)|*.*";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                currentItem = new ParseItemXML(dlg.FileName);
                itemtextBox.Text = currentItem.GetItemName();
                DescriptiontextBox.Text = currentItem.GetDescription();
                AssemblyNametextBox.Text = currentItem.GetAssemblyName();
                AssemblyLocationtextBox.Text = currentItem.GetAssemblyLocation();
                nativeImagetextBox.Text = currentItem.GetNativeImageLocation();

                injectionModeTextBox.Text = currentItem.GetInjectionMode();

                foreach (PayloadExternalRef externalRef in currentItem.GetRefernces())
                    ReferenceslistBox.Items.Add(externalRef.InjectedRefFile);
                foreach (PayloadFunction payloadFunction in currentItem.GetFunctions())
                    FunctionslistBox.Items.Add(payloadFunction.FunctionCodeFile);
                foreach (PayloadInjectedCode payloadCode in currentItem.GetInjectedPayloads())
                    PayloadslistBox.Items.Add(payloadCode.InjectedCodeFile);

                DisplayProgress("Item "+ currentItem.GetItemName() + " loaded successfully");
            }
        }

        private void aboutNETSploitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show(".NET-Sploit was developed by Erez Metula (erezmetula@gmail.com) as a PoC for .NET Framework modification ", ".NET-Spolit " + Configuration.version,
            MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private bool DecideCopy(string filename, string location)
        {
            string fullpath = location + "\\" + filename;
            if (Configuration.copyFromGAC)
            {
                if (FileUtilities.IsFileExists(Configuration.InputDir + "\\" + filename))
                {
                    if (MessageBox.Show("Overwrite exisiting file?", "Overwrite?", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                    {
                        FileUtilities.CopyFromGAC(filename, fullpath);
                    }
                }
                else
                    FileUtilities.CopyFromGAC(filename, fullpath);
            }
            else
            {
                if (!FileUtilities.IsFileExists(Configuration.InputDir + "\\" + filename))
                {
                    MessageBox.Show("The file " + filename + " not exists in your Input directory. Please choose \"Copy DLL From GAC\" in order to continue.");
                    return false;
                }
            }
            return true;
        }

        private bool CheckExternalExecutables()
        {
            if (!FileUtilities.IsFileExists(Configuration.ilasmLocation))
                if (MessageBox.Show("ilasm.exe not found. Continue anyway?", "ilasm.exe not found", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.No)
                {
                    MessageBox.Show("set the \"ilasmLocation\" section in Config.xml to the correct location of ilasm.exe");
                    return false;
                }

            if (!FileUtilities.IsFileExists(Configuration.ildasmLocation))
                if (MessageBox.Show("ildasm.exe not found. Continue anyway?", "ildasm.exe not found", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.No)
                {
                    MessageBox.Show("set the \"ildasmLocation\" section in Config.xml to the correct location of ildasm.exe (maybe the .NET Framework SDK is not installed)");
                    return false;
                }

            if (!FileUtilities.IsFileExists(Configuration.ngenLocation))
                if (MessageBox.Show("ngen.exe not found. Continue anyway?", "ngen.exe not found", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.No)
                {
                    MessageBox.Show("set the \"ngenLocation\" section in Config.xml to the correct location of ngen.exe");
                    return false;
                }
    
            return true;
        }

        private void startToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (currentItem == null)
            {
                MessageBox.Show("You should load an item first..");
                return;
            }

            if (!CheckExternalExecutables())
            {
                MessageBox.Show("Execution stopped. Modify Config.xml and run again.");
                return;
            }

            try
            {
                CodeChanger changer = new CodeChanger(currentItem.GetAssemblyName());
                if (!DecideCopy(currentItem.GetAssemblyName(), currentItem.GetAssemblyLocation()))
                    return;
                DisplayProgress("Decompiling " + currentItem.GetAssemblyName() + " (it may take a while..)");
                Cursor.Current = Cursors.WaitCursor;
                //decompile the assembly
                changer.Decompile();
                Cursor.Current = Cursors.Default;
                //inject all references
                foreach (PayloadExternalRef externalRef in currentItem.GetRefernces())
                {
                    DisplayProgress("Injecting required reference (" + externalRef.InjectedRefFile + ")");
                    changer.InjectExternalRef(externalRef);
                }
                //inject all functions
                foreach (PayloadFunction payloadFunction in currentItem.GetFunctions())
                {
                    DisplayProgress("Injecting Function (" + payloadFunction.FunctionCodeFile + ")");
                    changer.InjectFunction(payloadFunction);
                }
                //inject all payloads
                foreach (PayloadInjectedCode payloadCode in currentItem.GetInjectedPayloads())
                {
                    DisplayProgress("Injecting the payload (" + payloadCode.InjectedCodeFile + ")");

                    switch (payloadCode.InjectionMode)
                    {
                        case PayloadInjectedCode.InjectionModeEnum.PreAppend:
                            changer.InjectPayloadCode(payloadCode, payloadCode.InjectionMode);
                            break;
                        case PayloadInjectedCode.InjectionModeEnum.PostAppend:
                            changer.InjectPayloadCode(payloadCode, payloadCode.InjectionMode);
                            break;
                        case PayloadInjectedCode.InjectionModeEnum.Replace:
                            changer.ReplaceWithPayloadCode(payloadCode);
                            break;
                    }


                }
                DisplayProgress("Recompiling " + currentItem.GetAssemblyName() + " (it may take a while..)");
                Cursor.Current = Cursors.WaitCursor;
                //compile the modified code
                changer.Recompile();
                Cursor.Current = Cursors.Default;
                MessageBox.Show("Done!! Your modified DLL is ready to be deployed...", "Finish");
                DisplayProgress("Done!! Your modified DLL is ready to be deployed...");
                if (MessageBox.Show("Generate batch deployer?", "Deployer", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                {
                    DisplayProgress("Generating deployer/undeployer for " + currentItem.GetAssemblyName());
                    FileUtilities.GenerateDeployer(currentItem.GetAssemblyName(), currentItem.GetAssemblyLocation(), currentItem.GetNativeImageLocation());
                    DisplayProgress("Use " + Configuration.deployFileName + " to inject your modified DLL, and " + Configuration.undeployFileName + " to restore the original DLL");
                }
                DisplayProgress("Finished successfuly");
            }

            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void copyFromGACToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Configuration.copyFromGAC = !Configuration.copyFromGAC;
        }

        private void promptForDeployerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Configuration.promptForDeployer = !Configuration.promptForDeployer;
        }

        private void deleteProcessingDirectoryContentToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete the content of the directories " + Configuration.InputDir + "," + Configuration.DecompiledDir + "," + Configuration.OutputDir + "?", "Delete directory content", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
            {
                FileUtilities.DeleteDirContent(Configuration.InputDir);
                FileUtilities.DeleteDirContent(Configuration.DecompiledDir);
                FileUtilities.DeleteDirContent(Configuration.OutputDir);
                DisplayProgress("Directory content was cleaned");
            }
        }

       

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

               
    }    
}